////////////////////////////////////////////////////////////////
// ResponseNotifierPlugin.java
//
// Copyright (C) ObjectPlanet, Inc.
// All rights reserved.
// Confidential, unpublished property of ObjectPlanet, Inc.
////////////////////////////////////////////////////////////////


//package com.objectplanet.survey.plugin.ResponseNotifier;

import com.objectplanet.survey.plugin.IPBComponent;
import com.objectplanet.survey.plugin.Plugin;
import com.objectplanet.survey.plugin.PluginBusSurvey;
import com.objectplanet.survey.plugin.PluginBusAdmin;
import com.objectplanet.survey.plugin.event.*;
import com.objectplanet.survey.plugin.api.*;

import java.util.Properties;
import java.util.ArrayList;
import java.util.Map;
import java.util.StringTokenizer;
import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 *  The response notifier plugin sends email to one recipient, on various events
 *  in the survey module. For now, it supports response messages only.
 *
 *@author     Torgeir Punnerud
 *@created    29. august 2002
 */
public class ResponseNotifierPlugin extends Plugin implements IPBComponent {

	String fromEmail;
	String newRespondentMessage;
	boolean newRespondentOn;
	String newRespondentSubject;

	boolean newSurveyEventOn;
	boolean newUserEventOn;

	// helper variables
	Properties props;
	File propsFile;
	String respondentCompleteMessage;
	boolean respondentCompleteOn;
	String respondentCompleteSubject;
	boolean surveyDeletedEventOn;

	// Plugin state variables
	ArrayList surveyIds;
	String surveyIdsString;
	boolean userDeletedEventOn;

	// Default strings and messages messages
	private final static String PROCESS_NOK = "<br>The configuration values " +
			"could <b>not</b> be saved.<br>&nbsp;";

	private final static String PROCESS_OK = "<br>The ResponseNotifier setup is saved." +
			"<br>&nbsp;";

	private final static String FROM_EMAIL = "nobody@nobody.com";
	private final static String SURVEY_NAME = "[survey name]";
	private final static String SURVEY_ID = "[survey id]";
	private final static String USER_ID = "[user id]";
	private final static String USER_NAME = "[user name]";
	private final static String RESPONDENT_ID = "[respondent id]";

	private final static String DELETED_USER_NAME = "[user deleted name]";
	private final static String DELETED_USER_SUBJ = "User deleted";
	private final static String DELETED_USER_MSG = "A user was deleted \n\n" +
			"user: " + DELETED_USER_NAME + "\n" +
			"deleted by: " + USER_NAME + "\n";

	private final static String NEW_RESP_SUBJ = "New respondent";
	private final static String NEW_RESP_MSG = "<br>A new respondent has " +
			"started to respond to " +
			"survey \"[survey name]\".\n";


	private final static String NEW_SURVEY_SUBJ = "Survey created";
	private final static String NEW_SURVEY_MSG = "A survey was created. \n\n" +
			"survey: " + SURVEY_ID + "\n" +
			"created by: " + USER_NAME + "\n";

	private final static String NEW_USER_NAME = "[new user name]";
	private final static String NEW_USER_SUBJ = "New user created";
	private final static String NEW_USER_MSG = "A user was created\n\n" +
			"New user: " + NEW_USER_NAME + "\n" +
			"Created by: " + USER_NAME + "\n";


	private final static String RESP_COMPLETE_SUBJ = "Completed respondent";
	private final static String RESP_COMPLETE_MSG = "Respondent has completed \n\n" +
			"survey: \"" + SURVEY_NAME + "\"\n" +
			"Respondent id: " + RESPONDENT_ID + "\n\n";

	private final static String SURVEY_DELETED_SUBJ = "Survey deleted";
	private final static String SURVEY_DELETED_MSG = "A survey was deleted \n\n" +
			"survey: " + SURVEY_NAME + "\n" +
			"deleted by: " + USER_NAME + "\n";





	/**
	 *  Constructor for the ResponseNotifierPlugin object
	 */
	public ResponseNotifierPlugin() {
		super();
		props = new Properties();
		propsFile = new File(PluginUtil.getSystemHome() +
				"/WEB-INF/plugins/ResponseNotifier.properties");
		surveyIds = new ArrayList();
		surveyIdsString = "";
		newRespondentMessage = NEW_RESP_MSG;
		newRespondentSubject = NEW_RESP_SUBJ;
		respondentCompleteSubject = RESP_COMPLETE_SUBJ;
		respondentCompleteMessage = RESP_COMPLETE_MSG;
		respondentCompleteOn = false;
		newRespondentOn = false;

		newSurveyEventOn = false;
		surveyDeletedEventOn = false;
		newUserEventOn = false;
		userDeletedEventOn = false;

		fromEmail = FROM_EMAIL;
	}


	/**
	 *  Gets the HTML code needed to perform actions on the plugin. Actions are
	 *  things like reset, start/stop, release resources etc. It is entirely up to
	 *  the plugin how this is feature is implemented. It works exactly like the
	 *  getSetupHTML() method (see this method for more info).
	 *
	 *@return    The HTML code for running actions on this plugin.
	 */
	public String getActionHTML() {
		return "<br><br>Action screen for the Response notifier plugin<br><br>&nbsp;";
	}


	/**
	 *  Gets the HTML code for displaying help. It is not required, but
	 *  recommended. A plugin without help is most likely useless for everyone
	 *  except the developer.
	 *
	 *@return    The help HTML code
	 */
	public String getHelpHTML() {
		return "<br><br>Help screen for the Response notifier plugin " +
				"is not implemented yet<br><br>&nbsp;";
	}


	/**
	 *  Gets the HTML code needed to perform setup/config operations on this
	 *  plugin. It is entirely up to the plugin how this setup screen is built. It
	 *  is recommended that the look & feel of Opinio is used.<br>
	 *  <br>
	 *  The setup must be done with an HTML form. All the form elements/values will
	 *  be captured by Opinio on submit, and returned in an Map to the plugin
	 *  through the processSetup() method.<br>
	 *  <br>
	 *  This method is not required by the plugin. If not implemented, a standard
	 *  no-op text is returned.
	 *
	 *@return    The HTML code for setup of this plugin.
	 */
	public String getSetupHTML() {
		return "<FORM ACTION=\"plugin.do?action=process\" METHOD=POST>" +
				"<input type=hidden name=pluginName value=ResponseNotifierPlugin>" +
				"<table width=\"100%\">" +
		//Survey events
				"	<tr><td class=form colspan=2 height=5></td></tr>" +
				"	<tr><td class=listsubtitle colspan=2>Survey events</td></tr>" +
				"	<tr><td class=form colspan=2 height=5></td></tr>" +
				"	<tr><td class=form width=\"30%\"><nobr>" +
				"			Survey ids (separated by comma. 0=all):<nobr>" +
				"		</td>" +
				"		<td class=form width=\"70%\">" +
				"			<INPUT TYPE=\"text\" class=field400 size=40 " +
				"			name=\"surveyId\" value=\"" + surveyIdsString + "\">" +
				"		</td>" +
				"	</tr>" +
				"	<tr><td class=form>New respondent notification:</td>" +
				"		<td class=form>" +
				"			<INPUT TYPE=\"checkbox\" class=checkbox " +
				"			name=\"newRespondentOn\"" +
				getChecked(newRespondentOn) + ">" +
				"		</td>" +
				"	</tr>" +
				"	<tr><td class=form>New respondent message:</td>" +
				"		<td class=form>" +
				"			<TEXTAREA class=text400 cols=40 rows=4 " +
				"			name=\"newRespondentMessage\">" +
				newRespondentMessage + "</TEXTAREA>" +
				"		</td>" +
				"	</tr>" +
				"	<tr><td class=form>Respondent complete notification:</td>" +
				"		<td class=form>" +
				"			<INPUT TYPE=checkbox class=checkbox " +
				"			name=\"respondentCompleteOn\"" +
				getChecked(respondentCompleteOn) + ">" +
				"		</td>" +
				"	</tr>" +
				"	<tr><td class=form>Respondent complete message:</td>" +
				"		<td class=form>" +
				"			<TEXTAREA class=text400 cols=40 rows=4 " +
				"			name=\"respondentCompleteMessage\">" +
				respondentCompleteMessage + "</TEXTAREA>" +
				"		</td>" +
				"	</tr>" +
				"	<tr><td class=form>Email address:</td>" +
				"		<td class=form>" +
				"			<INPUT TYPE=\"text\" class=field400 size=40 " +
				"			name=\"email\" value=\"" + fromEmail + "\">" +
				"		</td>" +
				"	</tr>" +
		//Admin events
				"	<tr><td class=form height=10 colspan=2></td></tr>" +
				"	<tr><td class=listsubtitle colspan=2>Admin events</td></tr>" +
				"	<tr><td class=form colspan=2 height=5></td></tr>" +
				"	<tr><td class=form>New user notification:</td>" +
				"		<td class=form>" +
				"			<INPUT TYPE=checkbox class=checkbox " +
				"			name=\"newUserEventOn\"" +
				getChecked(newUserEventOn) + ">" +
				"		</td>" +
				"	</tr>" +
				"	<tr><td class=form>User delete notification:</td>" +
				"		<td class=form>" +
				"			<INPUT TYPE=checkbox class=checkbox " +
				"			name=\"userDeletedEventOn\"" +
				getChecked(userDeletedEventOn) + ">" +
				"		</td>" +
				"	</tr>" +
				"	<tr><td class=form>New survey notification:</td>" +
				"		<td class=form>" +
				"			<INPUT TYPE=checkbox class=checkbox " +
				"			name=\"newSurveyEventOn\"" +
				getChecked(newSurveyEventOn) + ">" +
				"		</td>" +
				"	</tr>" +
				"	<tr><td class=form>Survey delete notification:</td>" +
				"		<td class=form>" +
				"			<INPUT TYPE=checkbox class=checkbox " +
				"			name=\"surveyDeletedEventOn\"" +
				getChecked(surveyDeletedEventOn) + ">" +
				"		</td>" +
				"	</tr>" +
				"	<tr><td class=form colspan=2 height=5></td></tr></table>" +
		//Save button
				"<center>" +
				"<table width=\"100%\">" +
				"	<tr><td align=center class=listitemspace>" +
				"			<INPUT TYPE=submit class=button name=submitButton value=\"Save\">" +
				"		</td>" +
				"	</tr>" +
				"</table>" +
				"</center>";
	}


	/**
	 *  Handles the events that this plugin is registered to receive
	 *
	 *@param  event  The event.
	 */
	public void handleEvent(PluginBusEvent event) {
		if (event instanceof NewRespondentEvent && newRespondentOn) {
			NewRespondentEvent nrEvent = (NewRespondentEvent) event;
			sendEmail(
					NEW_RESP_SUBJ,
					getSurveyMessage(newRespondentMessage,
					nrEvent.getSurveyId(),
					nrEvent.getRespondentId()));
		} else if (event instanceof RespondentCompleteEvent && respondentCompleteOn) {
			RespondentCompleteEvent rcEvent = (RespondentCompleteEvent) event;
			sendEmail(
					RESP_COMPLETE_SUBJ,
					getSurveyMessage(respondentCompleteMessage,
						rcEvent.getSurveyId(),
						rcEvent.getRespondentId()));
		} else if (event instanceof NewSurveyEvent && newSurveyEventOn) {
			NewSurveyEvent nsEvent = (NewSurveyEvent) event;
			sendEmail(NEW_SURVEY_SUBJ, getAdminMessage(nsEvent));
		} else if (event instanceof SurveyDeletedEvent && surveyDeletedEventOn) {
			SurveyDeletedEvent sdEvent = (SurveyDeletedEvent) event;
			sendEmail(SURVEY_DELETED_SUBJ, getAdminMessage(sdEvent));
		} else if (event instanceof NewUserEvent && newUserEventOn) {
			NewUserEvent nuEvent = (NewUserEvent) event;
			sendEmail(NEW_USER_SUBJ, getAdminMessage(nuEvent));
		} else if (event instanceof UserDeletedEvent && userDeletedEventOn) {
			UserDeletedEvent udEvent = (UserDeletedEvent) event;
			sendEmail(DELETED_USER_SUBJ, getAdminMessage(udEvent));
		}
	}


	/**
	 *  Method for processing the setup screen. The values from the setup screen is
	 *  collected by Opinio and passed to this method in a Map. It is up to the
	 *  implementor (developer) of this method to process the values appropriately,
	 *  and to persist the values if necessary.
	 *
	 *@param  values  The values entered by the user in the setup screen.
	 *@return         The response message, with info about the result of setup
	 *      values processing.
	 */
	public String processSetup(Map values) {
		// set the properties
		// Survey ids (surveyId)
		String[] sArray = (String[]) values.get("surveyId");
		String ids = null;
		if (sArray != null) {
			ids = sArray[0];
			props.setProperty("surveyId", ids);
		}

		// TODO: register the plugin for the surveys set

		// newRespondent
		sArray = (String[]) values.get("newRespondentOn");
		String newRespOn = null;
		if (sArray != null) {
			newRespOn = sArray[0];
		}

		if (newRespOn != null && !newRespOn.equals("")) {
			props.setProperty("newRespondentOn", "checked");
		} else {
			props.setProperty("newRespondentOn", "");
		}
		// newRespondentMessage
		sArray = (String[]) values.get("newRespondentMessage");
		if (sArray != null) {
			String newRespMsg = sArray[0];
			props.setProperty("newRespondentMessage", newRespMsg);
		}

		// respondentComplete
		sArray = (String[]) values.get("respondentCompleteOn");
		String respCompleteOn = null;
		if (sArray != null) {
			respCompleteOn = sArray[0];
		}
		if (respCompleteOn != null && !respCompleteOn.equals("")) {
			props.setProperty("respondentCompleteOn", "checked");
		} else {
			props.setProperty("respondentCompleteOn", "");
		}

		// respondentCompleteMessage
		sArray = (String[]) values.get("respondentCompleteMessage");
		String respCompleteMsg = null;
		if (sArray != null) {
			respCompleteMsg = sArray[0];
		}
		props.setProperty("respondentCompleteMessage", respCompleteMsg);

		// new user event
		sArray = (String[]) values.get("newUserEventOn");
		String newUserEventOnStr = null;
		if (sArray != null) {
			newUserEventOnStr = sArray[0];
		}
		if (newUserEventOnStr != null && !newUserEventOnStr.equals("")) {
			props.setProperty("newUserEventOn", "checked");
		} else {
			props.setProperty("newUserEventOn", "");
		}

		// user delete event
		sArray = (String[]) values.get("userDeletedEventOn");
		String userDeletedEventOnStr = null;
		if (sArray != null) {
			userDeletedEventOnStr = sArray[0];
		}
		if (userDeletedEventOnStr != null && !userDeletedEventOnStr.equals("")) {
			props.setProperty("userDeletedEventOn", "checked");
		} else {
			props.setProperty("userDeletedEventOn", "");
		}

		// new survey event
		sArray = (String[]) values.get("newSurveyEventOn");
		String newSurveyEventOnStr = null;
		if (sArray != null) {
			newSurveyEventOnStr = sArray[0];
		}
		if (newSurveyEventOnStr != null && !newSurveyEventOnStr.equals("")) {
			props.setProperty("newSurveyEventOn", "checked");
		} else {
			props.setProperty("newSurveyEventOn", "");
		}

		// survey delete event
		sArray = (String[]) values.get("surveyDeletedEventOn");
		String surveyDeletedEventOnStr = null;
		if (sArray != null) {
			surveyDeletedEventOnStr = sArray[0];
		}
		if (surveyDeletedEventOnStr != null && !surveyDeletedEventOnStr.equals("")) {
			props.setProperty("surveyDeletedEventOn", "checked");
		} else {
			props.setProperty("surveyDeletedEventOn", "");
		}

		// from email address
		sArray = (String[]) values.get("email");
		String fromEmailString = null;
		if (sArray != null) {
			fromEmailString = sArray[0];
		}
		props.setProperty("fromEmail", fromEmailString);

		// set the plugin attributes
		setAttributes();

		// save the properties (make them persistent)
		try {
			PluginUtil.storeProps(props, propsFile);
		} catch (IOException e) {
			errorMsg = PROCESS_NOK;
			errorMsg = errorMsg + "<br><br>" + e.getMessage();
		}
		return PROCESS_OK;
	}


	/**
	 *  Starts the plugin. The survey event bus registrations are made here. Any
	 *  defaults, plus whatever event settings are made through the setup screen
	 *  are read here.
	 */
	public void start() {
		// get the properties file and store the values in a local properties
		// variable.
		try {
			FileInputStream in = new FileInputStream(propsFile);
			BufferedInputStream buf = new BufferedInputStream((InputStream) in);
			props.load(buf);
			in.close();
		} catch (FileNotFoundException fe) {
			PluginUtil.log(propsFile.getAbsolutePath() +
					" file not found: "
					, PluginUtil.LOGGING_LEVEL_ERROR);
			return;
		} catch (IOException ie) {
			PluginUtil.log(propsFile.getAbsolutePath() +
					"/PluginNotifier.properties file could not be loaded: " +
					ie.getMessage(), PluginUtil.LOGGING_LEVEL_ERROR);
			return;
		}

		setAttributes();
		// the plugin should get the surveys that it wants to receive events for
		// Register to listen to all surveys for now

		PluginUtil.log("ResponseNotifierPlugin loaded", PluginUtil.LOGGING_LEVEL_MESSAGE);

	}


	/**
	 *  Method called by Opinio before exiting. Usually, nothing needs to be done
	 *  here. If the plugin holds any resources, release them here.
	 */
	public void stop() { }


	/**
	 *  Sets the attributes attribute of the ResponseNotifierPlugin object
	 */
	private void setAttributes() {
		newRespondentMessage = props.getProperty("newRespondentMessage");
		if (newRespondentMessage == null) {
			newRespondentMessage = "";
		}

		respondentCompleteMessage = props.getProperty("respondentCompleteMessage");
		if (respondentCompleteMessage == null) {
			respondentCompleteMessage = "";
		}

		fromEmail = props.getProperty("fromEmail");
		if (fromEmail == null) {
			fromEmail = FROM_EMAIL;
		}

		String respCompleteOn = props.getProperty("respondentCompleteOn");
		if (respCompleteOn == null || respCompleteOn.equals("")) {
			respondentCompleteOn = false;
		} else {
			respondentCompleteOn = true;
		}

		String newRespOn = props.getProperty("newRespondentOn");
		if (newRespOn == null || newRespOn.equals("")) {
			newRespondentOn = false;
		} else {
			newRespondentOn = true;
		}

		String newUserEventOnStr = props.getProperty("newUserEventOn");
		if (newUserEventOnStr == null || newUserEventOnStr.equals("")) {
			newUserEventOn = false;
		} else {
			newUserEventOn = true;
		}

		String userDeletedEventOnStr = props.getProperty("userDeletedEventOn");
		if (userDeletedEventOnStr == null || userDeletedEventOnStr.equals("")) {
			userDeletedEventOn = false;
		} else {
			userDeletedEventOn = true;
		}

		String newSurveyEventOnStr = props.getProperty("newSurveyEventOn");
		if (newSurveyEventOnStr == null || newSurveyEventOnStr.equals("")) {
			newSurveyEventOn = false;
		} else {
			newSurveyEventOn = true;
		}

		String surveyDeletedEventOnStr = props.getProperty("surveyDeletedEventOn");
		if (surveyDeletedEventOnStr == null || surveyDeletedEventOnStr.equals("")) {
			surveyDeletedEventOn = false;
		} else {
			surveyDeletedEventOn = true;
		}

		PluginBusAdmin.removeFromBus(this);
		if (newUserEventOn || userDeletedEventOn || newSurveyEventOn || surveyDeletedEventOn) {
			PluginBusAdmin.addToBus(this);
		}

		surveyIdsString = props.getProperty("surveyId");
		if (surveyIdsString == null) {
			surveyIdsString = "";
		}

		// unregister any registered id's
		for (int i = 0; i < surveyIds.size(); i++) {
			Long l = (Long) surveyIds.get(i);
			PluginBusSurvey.removeFromBus(this, l.intValue());
		}

		// put the new ids in an ArrayList
		surveyIds = getLongValues(surveyIdsString);

		// get the new survey id string, and put them back into properties
		StringBuffer ids = new StringBuffer();
		for (int i = 0; i < surveyIds.size(); i++) {
			Long l = (Long) surveyIds.get(i);
			ids.append(l.toString());
			if (i < surveyIds.size() - 1) {
				ids.append(",");
			}
		}
		surveyIdsString = ids.toString();

		// register to listen to new surveys
		for (int i = 0; i < surveyIds.size(); i++) {
			Long l = (Long) surveyIds.get(i);
			PluginBusSurvey.addToBus(this, l.intValue());
		}
	}


	/**
	 *  Prepares an new survey message for sending email.
	 *
	 *@param  event    Description of the Parameter
	 *@return          The message, prepared.
	 */
	private String getAdminMessage(AdminEvent event) {
		String message = "";
		User user = getUser(event.getUserId());
		Survey survey = null;
		long surveyId;
		StringBuffer buf = null;
		String findString;
		String replaceString;

		if (event instanceof NewSurveyEvent) {
			message = NEW_SURVEY_MSG;
			NewSurveyEvent nEvent = (NewSurveyEvent) event;
			message = prepareAdminMessage(message, user);
			// insert survey name
			findString = SURVEY_ID;
			surveyId = nEvent.getSurveyId();
			buf = new StringBuffer(message);
			replaceString = surveyId + "";
			replace(findString, replaceString, buf);
		} else if (event instanceof SurveyDeletedEvent) {
			message = SURVEY_DELETED_MSG;
			SurveyDeletedEvent sEvent = (SurveyDeletedEvent) event;
			message = prepareAdminMessage(message, user);
			// insert survey id
			findString = SURVEY_NAME;
			surveyId = sEvent.getSurveyId();
			try {
				survey = SurveyManager.getSurvey(PluginUtil.getSystemUser(), surveyId);
				replaceString = survey.getSurveyName();
				buf = new StringBuffer(message);
				replace(findString, replaceString, buf);
			} catch (SurveySystemException e) {
				return message;
			} catch (SurveySecurityException e) {
				return message;
			}
		} else if (event instanceof NewUserEvent) {
			message = NEW_USER_MSG;
			NewUserEvent nEvent = (NewUserEvent) event;
			message = prepareAdminMessage(message, user);
			// insert survey id
			findString = NEW_USER_NAME;
			replaceString = getLogin(nEvent.getNewUserId());

			buf = new StringBuffer(message);
			replace(findString, replaceString, buf);
		} else if (event instanceof UserDeletedEvent) {
			message = DELETED_USER_MSG;
			buf = new StringBuffer(message);
			UserDeletedEvent dEvent = (UserDeletedEvent) event;
			message = prepareAdminMessage(message, user);
			// insert survey id
			findString = USER_NAME;
			replaceString = user.getLogin();
			replace(findString, replaceString, buf);

			findString = DELETED_USER_NAME;
			replaceString = dEvent.getDeletedUserLogin();
			replace(findString, replaceString, buf);
		}

		if (buf != null) {
			return buf.toString();
		} else {
			return message;
		}
	}


	/**
	 *  Gets the checked attribute of the ResponseNotifierPlugin object
	 *
	 *@param  checked  Description of the Parameter
	 *@return          The checked value
	 */
	private String getChecked(boolean checked) {
		if (checked) {
			return " checked";
		} else {
			return "";
		}
	}


	/**
	 *  Gets the longValues attribute of the ResponseNotifierPlugin object
	 *
	 *@param  string  The string with numbers, comma separated.
	 *@return         The longValues value
	 */
	private ArrayList getLongValues(String string) {
		ArrayList longValues = new ArrayList();

		// populate the arraylist of surveyIds
		// tokenize the strings in the parameter
		if (string != null && !(string.equals(""))) {
			// if the string starts with a comma, it has an empty value
			// but the string tokenizer does not count this first comma.
			boolean empty_first = string.trim().startsWith(",");
			// get the values
			StringTokenizer st = new StringTokenizer(string, ",");

			// create all the long values
			while (st.hasMoreElements()) {
				try {
					longValues.add(new Long(st.nextToken().trim()));
				} catch (NumberFormatException e) {}
			}
		}
		return longValues;
	}


	/**
	 *  Notification messages contain survey names, respondent ids etc. These are
	 *  dynamic, and needs to be inserted into the messages, for every message
	 *  sent.
	 *
	 *@param  message       The message to prepare
	 *@param  surveyId      Survey id
	 *@param  respondentId  Respondent id
	 *@return               The message, prepared.
	 */
	private String getSurveyMessage(String message, long surveyId, long respondentId) {
		if (message == null) {
			return null;
		}

		StringBuffer buf = new StringBuffer(message);
		try{
			Survey survey = SurveyManager.getSurvey(PluginUtil.getSystemUser(), surveyId);
			replace(SURVEY_NAME, survey.getSurveyName(), buf);
			replace(RESPONDENT_ID, "" + respondentId, buf);
			return buf.toString();
		} catch (SurveySystemException e) {
			return message;
		} catch (SurveySecurityException e) {
			return message;
		}
	}


	/**
	 * Gets user object. Returns null if exception occurs.
	 */
	private User getUser(long userId) {
		try {
			User user = UserManager.getUser(PluginUtil.getSystemUser(), userId);
			return user;
		} catch (SurveySystemException e) {
			return null;
		} catch (SurveySecurityException e) {
			return null;
		}
	}


	/**
	 * Gets user object. Returns null if exception occurs.
	 */
	private String getLogin(long userId) {
		User user = getUser(userId);
		if(user != null){
			return user.getLogin();
		}
		return "Unknown";
	}


	/**
	 *  Prepares an admin message for sending email. The user name is inserted into
	 *  the message, if it contains the string USER_NAME (constant).
	 *
	 *@param  message  The message to prepare
	 *@param  user     User that performed the action
	 *@return          The message, prepared.
	 */
	private String prepareAdminMessage(String message, User user) {
		if (message == null || user == null) {
			return message;
		}

		StringBuffer buf = new StringBuffer(message);
		String findString;
		String replaceString = "Unknown";

		// insert user name in message
		findString = USER_NAME;
		if(user != null){
			replaceString = user.getLogin();
		}
		replace(findString, replaceString, buf);

		return buf.toString();
	}


	/**
	 *  Replaces all occurrences of a string with another string in a string
	 *  buffer.
	 *
	 *@param  s    String to search for
	 *@param  s2   String to replace with
	 *@param  buf  The string buffer to perform the search/replace operation on.
	 */
	private void replace(String s, String s2, StringBuffer buf) {
		int length = s.length();
		int position;
		String bufString = buf.toString();
		position = bufString.indexOf(s, 0);

		while (position > -1) {
			buf.replace(position, position + length, s2);
			bufString = buf.toString();
			position = bufString.indexOf(s, position + length);
		}
	}


	/**
	 *  Sends the notification about the events registered.
	 *
	 *@param  subject  Subject of the notification
	 *@param  message  The message to send
	 */
	private void sendEmail(String subject, String message) {
		// TODO: make the email and name configurable
		String fromName = fromEmail;
		try {
			PluginUtil.sendEmail(fromName, fromEmail, fromEmail, subject, message, "text/html");
		} catch (Exception e) {
			PluginUtil.log(e);
		}
	}
}

